@@ -21,10 +21,10 @@ import (
 	"github.com/containers/image/v5/types"
 	encconfig "github.com/containers/ocicrypt/config"
 	enchelpers "github.com/containers/ocicrypt/helpers"
-	"github.com/containers/podman/v4/cmd/podman/registry"
-	"github.com/containers/podman/v4/cmd/podman/utils"
-	"github.com/containers/podman/v4/pkg/domain/entities"
-	"github.com/containers/podman/v4/pkg/env"
+	"github.com/containers/podman/v5/cmd/podman/registry"
+	"github.com/containers/podman/v5/cmd/podman/utils"
+	"github.com/containers/podman/v5/pkg/domain/entities"
+	"github.com/containers/podman/v5/pkg/env"
 	"github.com/sirupsen/logrus"
 	"github.com/spf13/cobra"
 )
@@ -46,7 +46,13 @@ type BuildFlagsWrapper struct {
 	Cleanup bool
 }
 
-func DefineBuildFlags(cmd *cobra.Command, buildOpts *BuildFlagsWrapper) {
+// FarmBuildHiddenFlags are the flags hidden from the farm build command because they are either not
+// supported or don't make sense in the farm build use case
+var FarmBuildHiddenFlags = []string{"arch", "all-platforms", "compress", "cw", "disable-content-trust",
+	"logsplit", "manifest", "os", "output", "platform", "sign-by", "signature-policy", "stdin",
+	"variant"}
+
+func DefineBuildFlags(cmd *cobra.Command, buildOpts *BuildFlagsWrapper, isFarmBuild bool) {
 	flags := cmd.Flags()
 
 	// buildx build --load ignored, but added for compliance
@@ -65,11 +71,11 @@ func DefineBuildFlags(cmd *cobra.Command, buildOpts *BuildFlagsWrapper) {
 
 	// --pull flag
 	flag := budFlags.Lookup("pull")
-	if err := flag.Value.Set("true"); err != nil {
-		logrus.Errorf("Unable to set --pull to true: %v", err)
+	flag.DefValue = "missing"
+	if err := flag.Value.Set("missing"); err != nil {
+		logrus.Errorf("Unable to set --pull to 'missing': %v", err)
 	}
-	flag.DefValue = "true"
-	flag.Usage = "Always attempt to pull the image (errors are fatal)"
+	flag.Usage = `Pull image policy ("always/true"|"missing"|"never/false"|"newer")`
 	flags.AddFlagSet(&budFlags)
 
 	// Add the completion functions
@@ -116,6 +122,11 @@ func DefineBuildFlags(cmd *cobra.Command, buildOpts *BuildFlagsWrapper) {
 		_ = flags.MarkHidden("logsplit")
 		_ = flags.MarkHidden("cw")
 	}
+	if isFarmBuild {
+		for _, f := range FarmBuildHiddenFlags {
+			_ = flags.MarkHidden(f)
+		}
+	}
 }
 
 func ParseBuildOpts(cmd *cobra.Command, args []string, buildOpts *BuildFlagsWrapper) (*entities.BuildOptions, error) {
@@ -241,6 +252,7 @@ func ParseBuildOpts(cmd *cobra.Command, args []string, buildOpts *BuildFlagsWrap
 	}
 	apiBuildOpts.BuildOptions = *buildahDefineOpts
 	apiBuildOpts.ContainerFiles = containerFiles
+	apiBuildOpts.Authfile = buildOpts.Authfile
 
 	return &apiBuildOpts, err
 }
@@ -298,7 +310,9 @@ func buildFlagsWrapperToOptions(c *cobra.Command, contextDir string, flags *Buil
 		pullPolicy = buildahDefine.PullAlways
 	}
 
-	if flags.PullNever || strings.EqualFold(strings.TrimSpace(flags.Pull), "never") {
+	if flags.PullNever ||
+		strings.EqualFold(strings.TrimSpace(flags.Pull), "false") ||
+		strings.EqualFold(strings.TrimSpace(flags.Pull), "never") {
 		pullPolicy = buildahDefine.PullNever
 	}
 
@@ -322,15 +336,15 @@ func buildFlagsWrapperToOptions(c *cobra.Command, contextDir string, flags *Buil
 	}
 	if c.Flag("build-arg").Changed {
 		for _, arg := range flags.BuildArg {
-			av := strings.SplitN(arg, "=", 2)
-			if len(av) > 1 {
-				args[av[0]] = av[1]
+			key, val, hasVal := strings.Cut(arg, "=")
+			if hasVal {
+				args[key] = val
 			} else {
 				// check if the env is set in the local environment and use that value if it is
-				if val, present := os.LookupEnv(av[0]); present {
-					args[av[0]] = val
+				if val, present := os.LookupEnv(key); present {
+					args[key] = val
 				} else {
-					delete(args, av[0])
+					delete(args, key)
 				}
 			}
 		}
@@ -439,15 +453,15 @@ func buildFlagsWrapperToOptions(c *cobra.Command, contextDir string, flags *Buil
 	additionalBuildContext := make(map[string]*buildahDefine.AdditionalBuildContext)
 	if c.Flag("build-context").Changed {
 		for _, contextString := range flags.BuildContext {
-			av := strings.SplitN(contextString, "=", 2)
-			if len(av) > 1 {
-				parseAdditionalBuildContext, err := parse.GetAdditionalBuildContext(av[1])
+			key, val, hasVal := strings.Cut(contextString, "=")
+			if hasVal {
+				parseAdditionalBuildContext, err := parse.GetAdditionalBuildContext(val)
 				if err != nil {
 					return nil, fmt.Errorf("while parsing additional build context: %w", err)
 				}
-				additionalBuildContext[av[0]] = &parseAdditionalBuildContext
+				additionalBuildContext[key] = &parseAdditionalBuildContext
 			} else {
-				return nil, fmt.Errorf("while parsing additional build context: %q, accepts value in the form of key=value", av)
+				return nil, fmt.Errorf("while parsing additional build context: %s, accepts value in the form of key=value", contextString)
 			}
 		}
 	}
@@ -543,6 +557,7 @@ func buildFlagsWrapperToOptions(c *cobra.Command, contextDir string, flags *Buil
 		Target:                  flags.Target,
 		TransientMounts:         flags.Volumes,
 		UnsetEnvs:               flags.UnsetEnvs,
+		UnsetLabels:             flags.UnsetLabels,
 	}
 
 	if flags.IgnoreFile != "" {
