@@ -1,23 +1,24 @@
 //go:build amd64 || arm64
-// +build amd64 arm64
 
 package machine
 
 import (
+	"fmt"
 	"os"
 	"sort"
 	"strconv"
 	"time"
 
 	"github.com/containers/common/pkg/completion"
-	"github.com/containers/common/pkg/config"
 	"github.com/containers/common/pkg/report"
-	"github.com/containers/podman/v4/cmd/podman/common"
-	"github.com/containers/podman/v4/cmd/podman/registry"
-	"github.com/containers/podman/v4/cmd/podman/validate"
-	"github.com/containers/podman/v4/pkg/machine"
+	"github.com/containers/podman/v5/cmd/podman/common"
+	"github.com/containers/podman/v5/cmd/podman/registry"
+	"github.com/containers/podman/v5/cmd/podman/validate"
+	"github.com/containers/podman/v5/pkg/domain/entities"
+	"github.com/containers/podman/v5/pkg/machine"
+	"github.com/containers/podman/v5/pkg/machine/shim"
+	"github.com/containers/podman/v5/pkg/machine/vmconfigs"
 	"github.com/docker/go-units"
-	"github.com/pkg/errors"
 	"github.com/spf13/cobra"
 )
 
@@ -27,6 +28,7 @@ var (
 		Aliases:           []string{"ls"},
 		Short:             "List machines",
 		Long:              "List managed virtual machines.",
+		PersistentPreRunE: machinePreRunE,
 		RunE:              list,
 		Args:              validate.NoArgs,
 		ValidArgsFunction: completion.AutocompleteNone,
@@ -43,22 +45,6 @@ type listFlagType struct {
 	quiet     bool
 }
 
-type machineReporter struct {
-	Name           string
-	Default        bool
-	Created        string
-	Running        bool
-	LastUp         string
-	Stream         string
-	VMType         string
-	CPUs           uint64
-	Memory         string
-	DiskSize       string
-	Port           int
-	RemoteUsername string
-	IdentityPath   string
-}
-
 func init() {
 	registry.Commands = append(registry.Commands, registry.CliCommand{
 		Command: lsCmd,
@@ -67,27 +53,21 @@ func init() {
 
 	flags := lsCmd.Flags()
 	formatFlagName := "format"
-	flags.StringVar(&listFlag.format, formatFlagName, "{{.Name}}\t{{.VMType}}\t{{.Created}}\t{{.LastUp}}\t{{.CPUs}}\t{{.Memory}}\t{{.DiskSize}}\n", "Format volume output using JSON or a Go template")
-	_ = lsCmd.RegisterFlagCompletionFunc(formatFlagName, common.AutocompleteFormat(&machineReporter{}))
-	flags.BoolVar(&listFlag.noHeading, "noheading", false, "Do not print headers")
+	flags.StringVar(&listFlag.format, formatFlagName, "{{range .}}{{.Name}}\t{{.VMType}}\t{{.Created}}\t{{.LastUp}}\t{{.CPUs}}\t{{.Memory}}\t{{.DiskSize}}\n{{end -}}", "Format volume output using JSON or a Go template")
+	_ = lsCmd.RegisterFlagCompletionFunc(formatFlagName, common.AutocompleteFormat(&entities.ListReporter{}))
+	flags.BoolVarP(&listFlag.noHeading, "noheading", "n", false, "Do not print headers")
 	flags.BoolVarP(&listFlag.quiet, "quiet", "q", false, "Show only machine names")
 }
 
 func list(cmd *cobra.Command, args []string) error {
 	var (
-		opts         machine.ListOptions
-		listResponse []*machine.ListResponse
-		err          error
+		opts machine.ListOptions
+		err  error
 	)
 
-	if listFlag.quiet {
-		listFlag.format = "{{.Name}}\n"
-	}
-
-	provider := GetSystemDefaultProvider()
-	listResponse, err = provider.List(opts)
+	listResponse, err := shim.List([]vmconfigs.VMProvider{provider}, opts)
 	if err != nil {
-		return errors.Wrap(err, "error listing vms")
+		return err
 	}
 
 	// Sort by last run
@@ -99,12 +79,15 @@ func list(cmd *cobra.Command, args []string) error {
 		return listResponse[i].Running
 	})
 
-	if report.IsJSON(listFlag.format) {
-		machineReporter, err := toMachineFormat(listResponse)
-		if err != nil {
-			return err
-		}
+	defaultCon := ""
+	con, err := registry.PodmanConfig().ContainersConfDefaultsRO.GetConnection("", true)
+	if err == nil {
+		// ignore the error here we only want to know if we have a default connection to show it in list
+		defaultCon = con.Name
+	}
 
+	if report.IsJSON(listFlag.format) {
+		machineReporter := toMachineFormat(listResponse, defaultCon)
 		b, err := json.MarshalIndent(machineReporter, "", "    ")
 		if err != nil {
 			return err
@@ -113,53 +96,41 @@ func list(cmd *cobra.Command, args []string) error {
 		return nil
 	}
 
-	machineReporter, err := toHumanFormat(listResponse)
-	if err != nil {
-		return err
-	}
-
+	machineReporter := toHumanFormat(listResponse, defaultCon)
 	return outputTemplate(cmd, machineReporter)
 }
 
-func outputTemplate(cmd *cobra.Command, responses []*machineReporter) error {
-	headers := report.Headers(machineReporter{}, map[string]string{
+func outputTemplate(cmd *cobra.Command, responses []*entities.ListReporter) error {
+	headers := report.Headers(entities.ListReporter{}, map[string]string{
 		"LastUp":   "LAST UP",
 		"VmType":   "VM TYPE",
 		"CPUs":     "CPUS",
 		"Memory":   "MEMORY",
 		"DiskSize": "DISK SIZE",
 	})
-	printHeader := !listFlag.noHeading
-	if listFlag.quiet {
-		printHeader = false
-	}
-	var row string
+
+	rpt := report.New(os.Stdout, cmd.Name())
+	defer rpt.Flush()
+
+	var err error
 	switch {
-	case cmd.Flags().Changed("format"):
-		row = cmd.Flag("format").Value.String()
-		listFlag.noHeading = !report.HasTable(row)
-		row = report.NormalizeFormat(row)
+	case cmd.Flag("format").Changed:
+		rpt, err = rpt.Parse(report.OriginUser, listFlag.format)
+	case listFlag.quiet:
+		rpt, err = rpt.Parse(report.OriginUser, "{{.Name}}\n")
 	default:
-		row = cmd.Flag("format").Value.String()
+		rpt, err = rpt.Parse(report.OriginPodman, listFlag.format)
 	}
-	format := report.EnforceRange(row)
-
-	tmpl, err := report.NewTemplate("list").Parse(format)
 	if err != nil {
 		return err
 	}
 
-	w, err := report.NewWriterDefault(os.Stdout)
-	if err != nil {
-		return err
-	}
-	defer w.Flush()
-	if printHeader {
-		if err := tmpl.Execute(w, headers); err != nil {
-			return errors.Wrapf(err, "failed to write report column headers")
+	if rpt.RenderHeaders && !listFlag.noHeading {
+		if err := rpt.Execute(headers); err != nil {
+			return fmt.Errorf("failed to write report column headers: %w", err)
 		}
 	}
-	return tmpl.Execute(w, responses)
+	return rpt.Execute(responses)
 }
 
 func strTime(t time.Time) string {
@@ -181,16 +152,11 @@ func streamName(imageStream string) string {
 	return imageStream
 }
 
-func toMachineFormat(vms []*machine.ListResponse) ([]*machineReporter, error) {
-	cfg, err := config.ReadCustomConfig()
-	if err != nil {
-		return nil, err
-	}
-
-	machineResponses := make([]*machineReporter, 0, len(vms))
+func toMachineFormat(vms []*machine.ListResponse, defaultCon string) []*entities.ListReporter {
+	machineResponses := make([]*entities.ListReporter, 0, len(vms))
 	for _, vm := range vms {
-		response := new(machineReporter)
-		response.Default = vm.Name == cfg.Engine.ActiveService
+		response := new(entities.ListReporter)
+		response.Default = vm.Name == defaultCon
 		response.Name = vm.Name
 		response.Running = vm.Running
 		response.LastUp = strTime(vm.LastUp)
@@ -198,45 +164,48 @@ func toMachineFormat(vms []*machine.ListResponse) ([]*machineReporter, error) {
 		response.Stream = streamName(vm.Stream)
 		response.VMType = vm.VMType
 		response.CPUs = vm.CPUs
-		response.Memory = strUint(vm.Memory)
-		response.DiskSize = strUint(vm.DiskSize)
+		response.Memory = strUint(uint64(vm.Memory.ToBytes()))
+		response.DiskSize = strUint(uint64(vm.DiskSize.ToBytes()))
 		response.Port = vm.Port
 		response.RemoteUsername = vm.RemoteUsername
 		response.IdentityPath = vm.IdentityPath
+		response.Starting = vm.Starting
+		response.UserModeNetworking = vm.UserModeNetworking
 
 		machineResponses = append(machineResponses, response)
 	}
-	return machineResponses, nil
+	return machineResponses
 }
 
-func toHumanFormat(vms []*machine.ListResponse) ([]*machineReporter, error) {
-	cfg, err := config.ReadCustomConfig()
-	if err != nil {
-		return nil, err
-	}
-
-	humanResponses := make([]*machineReporter, 0, len(vms))
+func toHumanFormat(vms []*machine.ListResponse, defaultCon string) []*entities.ListReporter {
+	humanResponses := make([]*entities.ListReporter, 0, len(vms))
 	for _, vm := range vms {
-		response := new(machineReporter)
-		if vm.Name == cfg.Engine.ActiveService {
+		response := new(entities.ListReporter)
+		if vm.Name == defaultCon {
 			response.Name = vm.Name + "*"
 			response.Default = true
 		} else {
 			response.Name = vm.Name
 		}
-		if vm.Running {
+		switch {
+		case vm.Starting:
+			response.LastUp = "Currently starting"
+			response.Starting = true
+		case vm.Running:
 			response.LastUp = "Currently running"
 			response.Running = true
-		} else {
+		case vm.LastUp.IsZero():
+			response.LastUp = "Never"
+		default:
 			response.LastUp = units.HumanDuration(time.Since(vm.LastUp)) + " ago"
 		}
 		response.Created = units.HumanDuration(time.Since(vm.CreatedAt)) + " ago"
 		response.VMType = vm.VMType
 		response.CPUs = vm.CPUs
-		response.Memory = units.HumanSize(float64(vm.Memory))
-		response.DiskSize = units.HumanSize(float64(vm.DiskSize))
+		response.Memory = units.BytesSize(float64(vm.Memory.ToBytes()))
+		response.DiskSize = units.BytesSize(float64(vm.DiskSize.ToBytes()))
 
 		humanResponses = append(humanResponses, response)
 	}
-	return humanResponses, nil
+	return humanResponses
 }
