@@ -1,5 +1,4 @@
 //go:build amd64 || arm64
-// +build amd64 arm64
 
 package machine
 
@@ -7,11 +6,14 @@ import (
 	"fmt"
 	"net/url"
 
+	"github.com/containers/podman/v5/pkg/machine/define"
+	"github.com/containers/podman/v5/pkg/machine/env"
+
 	"github.com/containers/common/pkg/completion"
-	"github.com/containers/common/pkg/config"
-	"github.com/containers/podman/v4/cmd/podman/registry"
-	"github.com/containers/podman/v4/cmd/podman/utils"
-	"github.com/containers/podman/v4/pkg/machine"
+	"github.com/containers/podman/v5/cmd/podman/registry"
+	"github.com/containers/podman/v5/cmd/podman/utils"
+	"github.com/containers/podman/v5/pkg/machine"
+	"github.com/containers/podman/v5/pkg/machine/vmconfigs"
 	"github.com/spf13/cobra"
 )
 
@@ -44,24 +46,35 @@ func init() {
 	_ = sshCmd.RegisterFlagCompletionFunc(usernameFlagName, completion.AutocompleteNone)
 }
 
+// TODO Remember that this changed upstream and needs to updated as such!
+
 func ssh(cmd *cobra.Command, args []string) error {
 	var (
 		err     error
+		mc      *vmconfigs.MachineConfig
 		validVM bool
-		vm      machine.VM
 	)
 
+	dirs, err := env.GetMachineDirs(provider.VMType())
+	if err != nil {
+		return err
+	}
+
 	// Set the VM to default
 	vmName := defaultMachineName
-
 	// If len is greater than 0, it means we may have been
 	// provided the VM name.  If so, we check.  The VM name,
 	// if provided, must be in args[0].
 	if len(args) > 0 {
-		validVM, err = provider.IsValidVMName(args[0])
+		// note: previous incantations of this up by a specific name
+		// and errors were ignored.  this error is not ignored because
+		// it implies podman cannot read its machine files, which is bad
+		machines, err := vmconfigs.LoadMachinesInDir(dirs)
 		if err != nil {
 			return err
 		}
+
+		mc, validVM = machines[args[0]]
 		if validVM {
 			vmName = args[0]
 		} else {
@@ -79,9 +92,12 @@ func ssh(cmd *cobra.Command, args []string) error {
 		}
 	}
 
-	vm, err = provider.LoadVMByName(vmName)
-	if err != nil {
-		return fmt.Errorf("vm %s not found: %w", vmName, err)
+	// If the machine config was not loaded earlier, we load it now
+	if mc == nil {
+		mc, err = vmconfigs.LoadMachineByName(vmName, dirs)
+		if err != nil {
+			return fmt.Errorf("vm %s not found: %w", vmName, err)
+		}
 	}
 
 	if !validVM && sshOpts.Username == "" {
@@ -91,17 +107,30 @@ func ssh(cmd *cobra.Command, args []string) error {
 		}
 	}
 
-	err = vm.SSH(vmName, sshOpts)
+	state, err := provider.State(mc, false)
+	if err != nil {
+		return err
+	}
+	if state != define.Running {
+		return fmt.Errorf("vm %q is not running", mc.Name)
+	}
+
+	username := sshOpts.Username
+	if username == "" {
+		username = mc.SSH.RemoteUsername
+	}
+
+	err = machine.CommonSSH(username, mc.SSH.IdentityPath, mc.Name, mc.SSH.Port, sshOpts.Args)
 	return utils.HandleOSExecError(err)
 }
 
 func remoteConnectionUsername() (string, error) {
-	cfg, err := config.ReadCustomConfig()
+	con, err := registry.PodmanConfig().ContainersConfDefaultsRO.GetConnection("", true)
 	if err != nil {
 		return "", err
 	}
-	dest := cfg.Engine.ServiceDestinations[cfg.Engine.ActiveService].URI
-	uri, err := url.Parse(dest)
+
+	uri, err := url.Parse(con.URI)
 	if err != nil {
 		return "", err
 	}
