@@ -1,6 +1,7 @@
 package specgen
 
 import (
+	"errors"
 	"fmt"
 	"net"
 	"os"
@@ -8,13 +9,14 @@ import (
 
 	"github.com/containers/common/libnetwork/types"
 	"github.com/containers/common/pkg/cgroups"
-	"github.com/containers/podman/v4/libpod/define"
-	"github.com/containers/podman/v4/pkg/rootless"
-	"github.com/containers/podman/v4/pkg/util"
-	"github.com/containers/storage"
+	"github.com/containers/podman/v5/libpod/define"
+	"github.com/containers/podman/v5/pkg/namespaces"
+	"github.com/containers/podman/v5/pkg/rootless"
+	"github.com/containers/podman/v5/pkg/util"
+	storageTypes "github.com/containers/storage/types"
 	spec "github.com/opencontainers/runtime-spec/specs-go"
 	"github.com/opencontainers/runtime-tools/generate"
-	"github.com/pkg/errors"
+	"golang.org/x/exp/slices"
 )
 
 type NamespaceMode string
@@ -23,8 +25,7 @@ const (
 	// Default indicates the spec generator should determine
 	// a sane default
 	Default NamespaceMode = "default"
-	// Host means the the namespace is derived from
-	// the host
+	// Host means the namespace is derived from the host
 	Host NamespaceMode = "host"
 	// Path is the path to a namespace
 	Path NamespaceMode = "path"
@@ -40,10 +41,10 @@ const (
 	// None indicates the IPC namespace is created without mounting /dev/shm
 	None NamespaceMode = "none"
 	// NoNetwork indicates no network namespace should
-	// be joined.  loopback should still exists.
+	// be joined.  loopback should still exist.
 	// Only used with the network namespace, invalid otherwise.
 	NoNetwork NamespaceMode = "none"
-	// Bridge indicates that a CNI network stack
+	// Bridge indicates that the network backend (CNI/netavark)
 	// should be used.
 	// Only used with the network namespace, invalid otherwise.
 	Bridge NamespaceMode = "bridge"
@@ -51,6 +52,9 @@ const (
 	// be used.
 	// Only used with the network namespace, invalid otherwise.
 	Slirp NamespaceMode = "slirp4netns"
+	// Pasta indicates that a pasta network stack should be used.
+	// Only used with the network namespace, invalid otherwise.
+	Pasta NamespaceMode = "pasta"
 	// KeepId indicates a user namespace to keep the owner uid inside
 	// of the namespace itself.
 	// Only used with the user namespace, invalid otherwise.
@@ -155,21 +159,26 @@ func validateNetNS(n *Namespace) error {
 	switch n.NSMode {
 	case Slirp:
 		break
+	case Pasta:
+		if rootless.IsRootless() {
+			break
+		}
+		return fmt.Errorf("pasta networking is only supported for rootless mode")
 	case "", Default, Host, Path, FromContainer, FromPod, Private, NoNetwork, Bridge:
 		break
 	default:
-		return errors.Errorf("invalid network %q", n.NSMode)
+		return fmt.Errorf("invalid network %q", n.NSMode)
 	}
 
 	// Path and From Container MUST have a string value set
 	if n.NSMode == Path || n.NSMode == FromContainer {
 		if len(n.Value) < 1 {
-			return errors.Errorf("namespace mode %s requires a value", n.NSMode)
+			return fmt.Errorf("namespace mode %s requires a value", n.NSMode)
 		}
 	} else if n.NSMode != Slirp {
 		// All others except must NOT set a string value
 		if len(n.Value) > 0 {
-			return errors.Errorf("namespace value %s cannot be provided with namespace mode %s", n.Value, n.NSMode)
+			return fmt.Errorf("namespace value %s cannot be provided with namespace mode %s", n.Value, n.NSMode)
 		}
 	}
 
@@ -196,21 +205,21 @@ func (n *Namespace) validate() error {
 	switch n.NSMode {
 	case "", Default, Host, Path, FromContainer, FromPod, Private:
 		// Valid, do nothing
-	case NoNetwork, Bridge, Slirp:
-		return errors.Errorf("cannot use network modes with non-network namespace")
+	case NoNetwork, Bridge, Slirp, Pasta:
+		return errors.New("cannot use network modes with non-network namespace")
 	default:
-		return errors.Errorf("invalid namespace type %s specified", n.NSMode)
+		return fmt.Errorf("invalid namespace type %s specified", n.NSMode)
 	}
 
 	// Path and From Container MUST have a string value set
 	if n.NSMode == Path || n.NSMode == FromContainer {
 		if len(n.Value) < 1 {
-			return errors.Errorf("namespace mode %s requires a value", n.NSMode)
+			return fmt.Errorf("namespace mode %s requires a value", n.NSMode)
 		}
 	} else {
 		// All others must NOT set a string value
 		if len(n.Value) > 0 {
-			return errors.Errorf("namespace value %s cannot be provided with namespace mode %s", n.Value, n.NSMode)
+			return fmt.Errorf("namespace value %s cannot be provided with namespace mode %s", n.Value, n.NSMode)
 		}
 	}
 	return nil
@@ -221,29 +230,23 @@ func (n *Namespace) validate() error {
 // function.
 func ParseNamespace(ns string) (Namespace, error) {
 	toReturn := Namespace{}
-	switch {
-	case ns == "pod":
+	switch ns {
+	case "pod":
 		toReturn.NSMode = FromPod
-	case ns == "host":
+	case "host":
 		toReturn.NSMode = Host
-	case ns == "private", ns == "":
+	case "private", "":
 		toReturn.NSMode = Private
-	case strings.HasPrefix(ns, "ns:"):
-		split := strings.SplitN(ns, ":", 2)
-		if len(split) != 2 {
-			return toReturn, errors.Errorf("must provide a path to a namespace when specifying \"ns:\"")
-		}
-		toReturn.NSMode = Path
-		toReturn.Value = split[1]
-	case strings.HasPrefix(ns, "container:"):
-		split := strings.SplitN(ns, ":", 2)
-		if len(split) != 2 {
-			return toReturn, errors.Errorf("must provide name or ID or a container when specifying \"container:\"")
-		}
-		toReturn.NSMode = FromContainer
-		toReturn.Value = split[1]
 	default:
-		return toReturn, errors.Errorf("unrecognized namespace mode %s passed", ns)
+		if value, ok := strings.CutPrefix(ns, "ns:"); ok {
+			toReturn.NSMode = Path
+			toReturn.Value = value
+		} else if value, ok := strings.CutPrefix(ns, "container:"); ok {
+			toReturn.NSMode = FromContainer
+			toReturn.Value = value
+		} else {
+			return toReturn, fmt.Errorf("unrecognized namespace mode %s passed", ns)
+		}
 	}
 
 	return toReturn, nil
@@ -266,7 +269,7 @@ func ParseCgroupNamespace(ns string) (Namespace, error) {
 		case "private", "":
 			toReturn.NSMode = Private
 		default:
-			return toReturn, errors.Errorf("unrecognized cgroup namespace mode %s passed", ns)
+			return toReturn, fmt.Errorf("unrecognized cgroup namespace mode %s passed", ns)
 		}
 	} else {
 		toReturn.NSMode = Host
@@ -274,7 +277,7 @@ func ParseCgroupNamespace(ns string) (Namespace, error) {
 	return toReturn, nil
 }
 
-// ParseIPCNamespace parses a ipc namespace specification in string
+// ParseIPCNamespace parses an ipc namespace specification in string
 // form.
 func ParseIPCNamespace(ns string) (Namespace, error) {
 	toReturn := Namespace{}
@@ -293,85 +296,32 @@ func ParseIPCNamespace(ns string) (Namespace, error) {
 // form.
 func ParseUserNamespace(ns string) (Namespace, error) {
 	toReturn := Namespace{}
-	switch {
-	case ns == "auto":
+	switch ns {
+	case "auto":
 		toReturn.NSMode = Auto
 		return toReturn, nil
-	case strings.HasPrefix(ns, "auto:"):
-		split := strings.SplitN(ns, ":", 2)
-		if len(split) != 2 {
-			return toReturn, errors.Errorf("invalid setting for auto: mode")
-		}
-		toReturn.NSMode = Auto
-		toReturn.Value = split[1]
-		return toReturn, nil
-	case ns == "keep-id":
+	case "keep-id":
 		toReturn.NSMode = KeepID
 		return toReturn, nil
-	case ns == "nomap":
+	case "nomap":
 		toReturn.NSMode = NoMap
 		return toReturn, nil
-	case ns == "":
+	case "":
 		toReturn.NSMode = Host
 		return toReturn, nil
-	}
-	return ParseNamespace(ns)
-}
-
-// ParseNetworkNamespace parses a network namespace specification in string
-// form.
-// Returns a namespace and (optionally) a list of CNI networks to join.
-func ParseNetworkNamespace(ns string, rootlessDefaultCNI bool) (Namespace, map[string]types.PerNetworkOptions, error) {
-	toReturn := Namespace{}
-	networks := make(map[string]types.PerNetworkOptions)
-	// Net defaults to Slirp on rootless
-	switch {
-	case ns == string(Slirp), strings.HasPrefix(ns, string(Slirp)+":"):
-		toReturn.NSMode = Slirp
-	case ns == string(FromPod):
-		toReturn.NSMode = FromPod
-	case ns == "" || ns == string(Default) || ns == string(Private):
-		if rootless.IsRootless() {
-			if rootlessDefaultCNI {
-				toReturn.NSMode = Bridge
-			} else {
-				toReturn.NSMode = Slirp
-			}
-		} else {
-			toReturn.NSMode = Bridge
-		}
-	case ns == string(Bridge):
-		toReturn.NSMode = Bridge
-	case ns == string(NoNetwork):
-		toReturn.NSMode = NoNetwork
-	case ns == string(Host):
-		toReturn.NSMode = Host
-	case strings.HasPrefix(ns, "ns:"):
-		split := strings.SplitN(ns, ":", 2)
-		if len(split) != 2 {
-			return toReturn, nil, errors.Errorf("must provide a path to a namespace when specifying \"ns:\"")
-		}
-		toReturn.NSMode = Path
-		toReturn.Value = split[1]
-	case strings.HasPrefix(ns, string(FromContainer)+":"):
-		split := strings.SplitN(ns, ":", 2)
-		if len(split) != 2 {
-			return toReturn, nil, errors.Errorf("must provide name or ID or a container when specifying \"container:\"")
-		}
-		toReturn.NSMode = FromContainer
-		toReturn.Value = split[1]
 	default:
-		// Assume we have been given a list of CNI networks.
-		// Which only works in bridge mode, so set that.
-		networkList := strings.Split(ns, ",")
-		for _, net := range networkList {
-			networks[net] = types.PerNetworkOptions{}
+		if value, ok := strings.CutPrefix(ns, "auto:"); ok {
+			toReturn.NSMode = Auto
+			toReturn.Value = value
+			return toReturn, nil
+		} else if value, ok := strings.CutPrefix(ns, "keep-id:"); ok {
+			toReturn.NSMode = KeepID
+			toReturn.Value = value
+			return toReturn, nil
+		} else {
+			return ParseNamespace(ns)
 		}
-
-		toReturn.NSMode = Bridge
 	}
-
-	return toReturn, networks, nil
 }
 
 // ParseNetworkFlag parses a network string slice into the network options
@@ -390,29 +340,23 @@ func ParseNetworkFlag(networks []string) (Namespace, map[string]types.PerNetwork
 
 	switch {
 	case ns == string(Slirp), strings.HasPrefix(ns, string(Slirp)+":"):
-		parts := strings.SplitN(ns, ":", 2)
-		if len(parts) > 1 {
+		key, options, hasOptions := strings.Cut(ns, ":")
+		if hasOptions {
 			networkOptions = make(map[string][]string)
-			networkOptions[parts[0]] = strings.Split(parts[1], ",")
+			networkOptions[key] = strings.Split(options, ",")
 		}
 		toReturn.NSMode = Slirp
 	case ns == string(FromPod):
 		toReturn.NSMode = FromPod
 	case ns == "" || ns == string(Default) || ns == string(Private):
-		// Net defaults to Slirp on rootless
-		if rootless.IsRootless() {
-			toReturn.NSMode = Slirp
-			break
-		}
-		// if root we use bridge
-		fallthrough
+		toReturn.NSMode = Private
 	case ns == string(Bridge), strings.HasPrefix(ns, string(Bridge)+":"):
 		toReturn.NSMode = Bridge
-		parts := strings.SplitN(ns, ":", 2)
+		_, options, hasOptions := strings.Cut(ns, ":")
 		netOpts := types.PerNetworkOptions{}
-		if len(parts) > 1 {
+		if hasOptions {
 			var err error
-			netOpts, err = parseBridgeNetworkOptions(parts[1])
+			netOpts, err = parseBridgeNetworkOptions(options)
 			if err != nil {
 				return toReturn, nil, nil, err
 			}
@@ -425,37 +369,38 @@ func ParseNetworkFlag(networks []string) (Namespace, map[string]types.PerNetwork
 	case ns == string(Host):
 		toReturn.NSMode = Host
 	case strings.HasPrefix(ns, "ns:"):
-		split := strings.SplitN(ns, ":", 2)
-		if len(split) != 2 {
-			return toReturn, nil, nil, errors.Errorf("must provide a path to a namespace when specifying \"ns:\"")
-		}
+		_, value, _ := strings.Cut(ns, ":")
 		toReturn.NSMode = Path
-		toReturn.Value = split[1]
+		toReturn.Value = value
 	case strings.HasPrefix(ns, string(FromContainer)+":"):
-		split := strings.SplitN(ns, ":", 2)
-		if len(split) != 2 {
-			return toReturn, nil, nil, errors.Errorf("must provide name or ID or a container when specifying \"container:\"")
-		}
+		_, value, _ := strings.Cut(ns, ":")
 		toReturn.NSMode = FromContainer
-		toReturn.Value = split[1]
+		toReturn.Value = value
+	case ns == string(Pasta), strings.HasPrefix(ns, string(Pasta)+":"):
+		key, options, hasOptions := strings.Cut(ns, ":")
+		if hasOptions {
+			networkOptions = make(map[string][]string)
+			networkOptions[key] = strings.Split(options, ",")
+		}
+		toReturn.NSMode = Pasta
 	default:
 		// we should have a normal network
-		parts := strings.SplitN(ns, ":", 2)
-		if len(parts) == 1 {
+		name, options, hasOptions := strings.Cut(ns, ":")
+		if hasOptions {
+			if name == "" {
+				return toReturn, nil, nil, errors.New("network name cannot be empty")
+			}
+			netOpts, err := parseBridgeNetworkOptions(options)
+			if err != nil {
+				return toReturn, nil, nil, fmt.Errorf("invalid option for network %s: %w", name, err)
+			}
+			podmanNetworks[name] = netOpts
+		} else {
 			// Assume we have been given a comma separated list of networks for backwards compat.
 			networkList := strings.Split(ns, ",")
 			for _, net := range networkList {
 				podmanNetworks[net] = types.PerNetworkOptions{}
 			}
-		} else {
-			if parts[0] == "" {
-				return toReturn, nil, nil, errors.New("network name cannot be empty")
-			}
-			netOpts, err := parseBridgeNetworkOptions(parts[1])
-			if err != nil {
-				return toReturn, nil, nil, errors.Wrapf(err, "invalid option for network %s", parts[0])
-			}
-			podmanNetworks[parts[0]] = netOpts
 		}
 
 		// networks need bridge mode
@@ -464,27 +409,27 @@ func ParseNetworkFlag(networks []string) (Namespace, map[string]types.PerNetwork
 
 	if len(networks) > 1 {
 		if !toReturn.IsBridge() {
-			return toReturn, nil, nil, errors.Wrapf(define.ErrInvalidArg, "cannot set multiple networks without bridge network mode, selected mode %s", toReturn.NSMode)
+			return toReturn, nil, nil, fmt.Errorf("cannot set multiple networks without bridge network mode, selected mode %s: %w", toReturn.NSMode, define.ErrInvalidArg)
 		}
 
 		for _, network := range networks[1:] {
-			parts := strings.SplitN(network, ":", 2)
-			if parts[0] == "" {
-				return toReturn, nil, nil, errors.Wrapf(define.ErrInvalidArg, "network name cannot be empty")
+			name, options, hasOptions := strings.Cut(network, ":")
+			if name == "" {
+				return toReturn, nil, nil, fmt.Errorf("network name cannot be empty: %w", define.ErrInvalidArg)
 			}
-			if util.StringInSlice(parts[0], []string{string(Bridge), string(Slirp), string(FromPod), string(NoNetwork),
-				string(Default), string(Private), string(Path), string(FromContainer), string(Host)}) {
-				return toReturn, nil, nil, errors.Wrapf(define.ErrInvalidArg, "can only set extra network names, selected mode %s conflicts with bridge", parts[0])
+			if slices.Contains([]string{string(Bridge), string(Slirp), string(Pasta), string(FromPod), string(NoNetwork),
+				string(Default), string(Private), string(Path), string(FromContainer), string(Host)}, name) {
+				return toReturn, nil, nil, fmt.Errorf("can only set extra network names, selected mode %s conflicts with bridge: %w", name, define.ErrInvalidArg)
 			}
 			netOpts := types.PerNetworkOptions{}
-			if len(parts) > 1 {
+			if hasOptions {
 				var err error
-				netOpts, err = parseBridgeNetworkOptions(parts[1])
+				netOpts, err = parseBridgeNetworkOptions(options)
 				if err != nil {
-					return toReturn, nil, nil, errors.Wrapf(err, "invalid option for network %s", parts[0])
+					return toReturn, nil, nil, fmt.Errorf("invalid option for network %s: %w", name, err)
 				}
 			}
-			podmanNetworks[parts[0]] = netOpts
+			podmanNetworks[name] = netOpts
 		}
 	}
 
@@ -498,48 +443,48 @@ func parseBridgeNetworkOptions(opts string) (types.PerNetworkOptions, error) {
 	}
 	allopts := strings.Split(opts, ",")
 	for _, opt := range allopts {
-		split := strings.SplitN(opt, "=", 2)
-		switch split[0] {
+		name, value, _ := strings.Cut(opt, "=")
+		switch name {
 		case "ip", "ip6":
-			ip := net.ParseIP(split[1])
+			ip := net.ParseIP(value)
 			if ip == nil {
-				return netOpts, errors.Errorf("invalid ip address %q", split[1])
+				return netOpts, fmt.Errorf("invalid ip address %q", value)
 			}
 			netOpts.StaticIPs = append(netOpts.StaticIPs, ip)
 
 		case "mac":
-			mac, err := net.ParseMAC(split[1])
+			mac, err := net.ParseMAC(value)
 			if err != nil {
 				return netOpts, err
 			}
 			netOpts.StaticMAC = types.HardwareAddr(mac)
 
 		case "alias":
-			if split[1] == "" {
+			if value == "" {
 				return netOpts, errors.New("alias cannot be empty")
 			}
-			netOpts.Aliases = append(netOpts.Aliases, split[1])
+			netOpts.Aliases = append(netOpts.Aliases, value)
 
 		case "interface_name":
-			if split[1] == "" {
+			if value == "" {
 				return netOpts, errors.New("interface_name cannot be empty")
 			}
-			netOpts.InterfaceName = split[1]
+			netOpts.InterfaceName = value
 
 		default:
-			return netOpts, errors.Errorf("unknown bridge network option: %s", split[0])
+			return netOpts, fmt.Errorf("unknown bridge network option: %s", name)
 		}
 	}
 	return netOpts, nil
 }
 
-func SetupUserNS(idmappings *storage.IDMappingOptions, userns Namespace, g *generate.Generator) (string, error) {
+func SetupUserNS(idmappings *storageTypes.IDMappingOptions, userns Namespace, g *generate.Generator) (string, error) {
 	// User
 	var user string
 	switch userns.NSMode {
 	case Path:
 		if _, err := os.Stat(userns.Value); err != nil {
-			return user, errors.Wrap(err, "cannot find specified user namespace path")
+			return user, fmt.Errorf("cannot find specified user namespace path: %w", err)
 		}
 		if err := g.AddOrReplaceLinuxNamespace(string(spec.UserNamespace), userns.Value); err != nil {
 			return user, err
@@ -552,13 +497,18 @@ func SetupUserNS(idmappings *storage.IDMappingOptions, userns Namespace, g *gene
 			return user, err
 		}
 	case KeepID:
-		mappings, uid, gid, err := util.GetKeepIDMapping()
+		opts, err := namespaces.UsernsMode(userns.String()).GetKeepIDOptions()
+		if err != nil {
+			return user, err
+		}
+		mappings, uid, gid, err := util.GetKeepIDMapping(opts)
 		if err != nil {
 			return user, err
 		}
 		idmappings = mappings
 		g.SetProcessUID(uint32(uid))
 		g.SetProcessGID(uint32(gid))
+		g.AddProcessAdditionalGid(uint32(gid))
 		user = fmt.Sprintf("%d:%d", uid, gid)
 		if err := privateUserNamespace(idmappings, g); err != nil {
 			return user, err
@@ -571,6 +521,7 @@ func SetupUserNS(idmappings *storage.IDMappingOptions, userns Namespace, g *gene
 		idmappings = mappings
 		g.SetProcessUID(uint32(uid))
 		g.SetProcessGID(uint32(gid))
+		g.AddProcessAdditionalGid(uint32(gid))
 		user = fmt.Sprintf("%d:%d", uid, gid)
 		if err := privateUserNamespace(idmappings, g); err != nil {
 			return user, err
@@ -583,12 +534,12 @@ func SetupUserNS(idmappings *storage.IDMappingOptions, userns Namespace, g *gene
 	return user, nil
 }
 
-func privateUserNamespace(idmappings *storage.IDMappingOptions, g *generate.Generator) error {
+func privateUserNamespace(idmappings *storageTypes.IDMappingOptions, g *generate.Generator) error {
 	if err := g.AddOrReplaceLinuxNamespace(string(spec.UserNamespace), ""); err != nil {
 		return err
 	}
 	if idmappings == nil || (len(idmappings.UIDMap) == 0 && len(idmappings.GIDMap) == 0) {
-		return errors.Errorf("must provide at least one UID or GID mapping to configure a user namespace")
+		return errors.New("must provide at least one UID or GID mapping to configure a user namespace")
 	}
 	for _, uidmap := range idmappings.UIDMap {
 		g.AddLinuxUIDMapping(uint32(uidmap.HostID), uint32(uidmap.ContainerID), uint32(uidmap.Size))
